/*
*********************************************************************************************************
*                                              uC/OS-II
*                                        The Real-Time Kernel
*
*                    Copyright 1992-2021 Silicon Laboratories Inc. www.silabs.com
*
*                                 SPDX-License-Identifier: APACHE-2.0
*
*               This software is subject to an open source license and is distributed by
*                Silicon Laboratories Inc. pursuant to the terms of the Apache License,
*                    Version 2.0 available at www.apache.org/licenses/LICENSE-2.0.
*
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*
*                                    MCS51-STC32 Specific code
*
* Filename : os_cpu.h
* Version  : V2.93.01
*修改版本： V1.00
*修改时间： 2023-09-17
*作者：熊仔
*QQ：   616421684
*********************************************************************************************************
*/
#ifndef __OS_CPU_H__
#define __OS_CPU_H__

#include  "config.h"

#ifdef  OS_CPU_GLOBALS
#define OS_CPU_EXT
#else
#define OS_CPU_EXT  extern
#endif

/*************************************** 与编译器有关的数据类型 ***************************************/

typedef unsigned char  BOOLEAN;       //注意：不要使用bit定义，因为在结构体里无法使用
typedef unsigned char  INT8U;         //无符号8位数
typedef signed   char  INT8S;         //有符号8位数
typedef unsigned int   INT16U;        //无符号16位数
typedef signed   int   INT16S;        //有符号16位数
typedef unsigned long  INT32U;        //无符号32位数
typedef signed   long  INT32S;        //有符号32位数
typedef float          FP32;          //单精度浮点数
typedef double         FP64;          //双精度浮点数  51不支持双精度浮点数

typedef unsigned char  OS_STK;   //定义堆栈入口宽度为8位，指针变量是4字节。

typedef unsigned char  OS_CPU_SR;     // 定义CPU状态字的宽度为8位    
 

// ---------------------------------

/* 以下是兼容uC/OS v1.XX的数据类型，在uC/OS-II中没有使用  						*/

#define BYTE        INT8S
#define UBYTE       INT8U
#define WORD        INT16S
#define UWORD       INT16U
#define LONG        INT32S
#define ULONG       INT32U

/***************************************** 与处理器有关的代码 *****************************************/
//#define OS_CRITICAL_METHOD  3 
#define OS_CRITICAL_METHOD  0

// ---- 方法0 无临界区保护 ---------
#if OS_CRITICAL_METHOD==0
#define OS_ENTER_CRITICAL() 
#define OS_EXIT_CRITICAL()  
#endif

extern char data _bIE;  //切换任务前备份IE的值，最后退出的时候，恢复IE

#if OS_CRITICAL_METHOD==1
//但这样有一个问题，如果禁止中断的情况下调用OS功能函数，那么从功能函数返回时，中断可能变成允许的了，而实际上还是希望是禁止的。
#define OS_ENTER_CRITICAL() do{EA=0;}while(0) /* 直接禁止中断  */
#define OS_EXIT_CRITICAL()  do{EA=1;}while(0) /* 直接允许中断  */
#endif

#if OS_CRITICAL_METHOD==2 
//执行OS_ENTER_CRITICAL()时，首先执行JBC EA ，执行后自动EA清零，根据EA的情况，将中断状态保存到堆栈
//执行OS_EXIT_CRITICAL()时，再从堆栈中恢复原来的中断开/关状态。这种方法不会改变中断状态，避免前面的问题。 
//但是，当用户使用的处理器有堆栈指针相对寻址模式时，可能出现严重错误。
//问题分析：OS_ENTER_CRITICAL()时导致中断切换任务的时候硬件堆栈多入了一个_push_(IE)，
//解决方法：OSIntCtxSw()函数前，调用OS_EXIT_CRITICAL_NOT_INT()
#include <intrins.h>
//JBC指令方式
/*
#define OS_ENTER_CRITICAL() do{                             \
                                if(_testbit_(EA)){          \
                                    ACC = 0x80;             \
                                    _push_(ACC);            \
                                }else{                      \
                                    ACC = 0x00;             \
                                    _push_(ACC);            \
                                }                           \
                            }while(0)  
*/
//进一步优化
#define OS_ENTER_CRITICAL() do{                             \
                                ACC = 0x80;                 \
                                if(!_testbit_(EA)){         \
                                    ACC = 0x00;             \
                                }                           \
                                _push_(ACC);                \
                            }while(0)  
#define OS_EXIT_CRITICAL() do{ _pop_ (ACC);IE |= ACC;}while(0)          // 将IE从堆栈弹出，恢复IE值    

#endif

#if OS_CRITICAL_METHOD==3     
//如果需要在一个函数内嵌套使用临界区保护，需要用调用函数方法，不能用宏
//例子：
//1层 进入临界区
//cpu_sr1 = OS_CPU_SR_Save()；
//    2层 进入临界区
//    cpu_sr2 = OS_CPU_SR_Save()；
//.
//    2层 退出临界区
//    OS_CPU_SR_Restore(cpu_sr2)；
//1层 退出临界区
//OS_CPU_SR_Restore(cpu_sr1)；
//获取当前中断状态的值，并将其保存在C函数局部变量之中
//问题分析：可重入函数的局部变量cpu_sr会入仿真栈。导致中断切换任务的时候，仿真堆栈出问题。
//解决方法：OSIntCtxSw()函数前，调用OS_EXIT_CRITICAL_NOT_INT()，并且需要对仿真堆栈指针++操作。
extern OS_CPU_SR  OS_CPU_SR_Save(void);
extern void OS_CPU_SR_Restore(OS_CPU_SR cpu_sr);
extern OS_CRITICAL_METHOD_3_FIX(void);
#define  OS_ENTER_CRITICAL()  (cpu_sr = OS_CPU_SR_Save()) /*关闭中断*/
#define  OS_EXIT_CRITICAL()   (OS_CPU_SR_Restore(cpu_sr)) /*使能中断*/
#endif 

#if OS_CRITICAL_METHOD==4
void OSEnterCritical(void);
void OSEXitCritical(void);
void OSEXitCriticalNoINT(void);
#define OS_ENTER_CRITICAL()  OSEnterCritical()
#define OS_EXIT_CRITICAL()  OSEXitCritical()
#define OS_EXIT_CRITICAL_NOT_INT()   OSEXitCriticalNoINT()     /*保存状态到_bIE*/
#endif 

#define OS_STK_GROWTH   0 /* MCS-51堆栈从下往上增长，1=向下，0=向上   */


#define  OSCtxSw()      PendSv_GenerateSWInterrupt()    //任务切换宏
#define  OSIntCtxSw()   PendSv_GenerateSWInterrupt()    //任务切换宏
#define  OS_TASK_SW()   PendSv_GenerateSWInterrupt()    //任务切换宏


void Timer0_Init(void);

extern void          OSStartHighRdy          (void);
//extern void          OSIntCtxSw              (void);
//extern void          OSCtxSw                 (void);

#endif
